/***************************************************************************
 * Programa que implementa funcionalidades de administração e consultas a  *
 * um banco de dados de linhas de ônibus, o qual, além das linhas,         *
 * armazena empresas, horários, itinerários e logradouros.                 *
 ***************************************************************************
 * Desenvolvido por Adriano Pereira para a disciplina de Fundamentos de    *
 * Banco de Dados - II Sem 2008 - UFSM - Curso de Ciência da Computação    *
 ***************************************************************************
 * Adriano Pereira
 * 2701049
 * peradri@gmail.com
 */

import java.io.*;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;


/**
   Módulo administrativo.
   Classe que trata a interação com o usuário que irá manipular o
   banco de dados. Funciona como um "interpretador de comandos"
*/
public class Interacao{
   private AcessaBD db;
   private Empresa emp;
   private Linha ln;
   private Logradouro lg;
   private Horario hr;
   private Itinerario it;
   private int idEmpresa = -1;
   private int idLinha = -1;
   
   public Interacao(int a){
   }
   
   public Interacao(){
      try{
         db = new AcessaBD();
         emp = new Empresa(db);
         ln = new Linha(db);
         lg = new Logradouro(db);
         hr = new Horario(db);
         it = new Itinerario(db);
      }catch (Exception e){
         System.out.println("Nao foi possivel instanciar a classe Interacao! "+e);
      }
   }
   
   /**
      Lê uma linha de comando e chama o método "analisa" para analisá-la
   */
   public void interage(){
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try {
         String input;
         System.out.println("Quadro de horários de ônibus");
         System.out.println("Desenvolvido por Adriano Pereira");
         System.out.println("Para solicitar os comandos, digite 'ajuda'");
         do{
            System.out.print("# ");
            input = inReader.readLine();
            //System.out.println(input.compareTo("listar"));
            analisa(input);
         }while (!input.contentEquals("fim"));
      }catch (Exception e){
         System.out.println(e);
      }
   }
   
   /**
      Analisa uma linha de comando.
      @param input String contendo a linha digitada pelo usuário
   */
   public void analisa(String input){
   try{        
      if (input.regionMatches(0, "fim", 0, "fim".length()))
         return;
      else if (input.regionMatches(0, "ajuda", 0, "ajuda".length()))
         this.ajuda();
      else if (input.regionMatches(0, "listar", 0, "listar".length())){ // listar alguma coisa
         if (input.regionMatches("listar".length() + 1, "empresa", 0, "empresa".length())) // Empresas
            emp.listar();
         else if (input.regionMatches("listar".length() + 1, "linha", 0, "linha".length())) // Linhas
            ln.listar();
         else if (input.regionMatches("listar".length() + 1, "logradouro", 0, "logradouro".length())) // Logradouros
            lg.listar();
         else if (input.regionMatches("listar".length() + 1, "itinerario", 0, "itinerario".length())) // Itinerarios
            it.listar();
         else if (input.regionMatches("listar".length() + 1, "horario", 0, "horario".length())) // Horarios
            hr.listar();
      }else if (input.regionMatches(0, "inserir", 0, "inserir".length())){ // Inserir algo
         if (input.regionMatches("inserir".length() + 1, "empresa", 0, "empresa".length())) // Empresas
            this.inserirEmpresa(false);
         else if (input.regionMatches("inserir".length() + 1, "linha", 0, "linha".length())) // Linhas
            this.inserirLinha(false);
         else if (input.regionMatches("inserir".length() + 1, "logradouro", 0, "logradouro".length())) // Logradouros
            this.inserirLogradouro(false);
         else if (input.regionMatches("inserir".length() + 1, "itinerario", 0, "itinerario".length())) // Itinerarios
            this.inserirItinerario();
         else if (input.regionMatches("inserir".length() + 1, "horario", 0, "horario".length())) // Horarios
            this.inserirHorario(false);
      }else if (input.regionMatches(0, "editar", 0, "editar".length())){ // Editar 
         if (input.regionMatches("editar".length() + 1, "empresa", 0, "empresa".length())) // Empresas
            this.inserirEmpresa(true);
         else if (input.regionMatches("editar".length() + 1, "linha", 0, "linha".length())) // Linhas
            this.inserirLinha(true);
         else if (input.regionMatches("editar".length() + 1, "logradouro", 0, "logradouro".length())) // Logradouros
            this.inserirLogradouro(true);
         else if (input.regionMatches("editar".length() + 1, "horario", 0, "horario".length())) // Horarios
            this.inserirHorario(true);
      }else if (input.regionMatches(0, "remover", 0, "remover".length())){ // Remover
         if (input.regionMatches("remover".length() + 1, "empresa", 0, "empresa".length())) // Empresas
            this.removerEmpresa();
         else if (input.regionMatches("remover".length() + 1, "linha", 0, "linha".length())) // Linhas
            this.removerLinha();
         else if (input.regionMatches("remover".length() + 1, "logradouro", 0, "logradouro".length())) // Logradouros
            this.removerLogradouro();
         else if (input.regionMatches("remover".length() + 1, "horario", 0, "horario".length())) // Horarios
            this.removerHorario();
         else if (input.regionMatches("remover".length() + 1, "itinerario", 0, "itinerario".length())) // Itinerarios
            this.removerItinerario();

      }else if (input.regionMatches(0, "set", 0, "set".length())){ // "Selecionar"
         if (input.substring("set".length()+1,"set empresa".length()).contentEquals("empresa")) // Empresas
            this.setEmpresa(Integer.parseInt(input.substring("set empresa".length()+1)));
         else if (input.substring("set".length()+1,"set linha".length()).contentEquals("linha")) // Linha
            this.setLinha(Integer.parseInt(input.substring("set linha".length()+1)));

      }else{
         System.out.println("Comando não encontrado. Utilize 'ajuda' para obter uma lista dos comandos");
      }
   }catch(Exception e){}
   }
   
   /**
      Seleciona uma empresa ou exibe uma mensagem de erro (empresa não encontrada). 
      @param idEmpresa código da empresa a ser selecionada
   */
   public void setEmpresa(int idEmpresa){
      if(!emp.verifica(idEmpresa))
         System.out.println("Empresa não encontrada!");
      else
         this.idEmpresa = idEmpresa;
   }
   
   /**
      Seleciona uma linha ou exibe uma mensagem de erro (linha não encontrada). 
      @param idLinha código da linha a ser selecionada
   */
   
   public void setLinha(int idLinha){
      if(!ln.verifica(idLinha))
         System.out.println("Linha não encontrada!");
      else
         this.idLinha = idLinha;
   }
   
   
   /**
      Trata a inserção e edição de uma empresa.
      @param edit TRUE se edição, FALSE se inserção
   */
   public void inserirEmpresa(boolean edit){
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try {
         String id;
         if (edit){ // Edição. Pergunta o código da empresa
            do {
               System.out.println(":: Digite o código da empresa:");
               id = inReader.readLine();
            }while (!emp.verifica(Integer.parseInt(id)));
            try{
               this.idEmpresa = Integer.parseInt(id);
               emp.listar(this.idEmpresa);
            }catch(Exception e){
               System.out.println("Código inválido! Utilize # listar emprsas para descobrir o código.");
               return;
            }
         }
         // Requer os dados da empresa a ser inserida/editada
         System.out.println(":: Digite o nome da empresa:");
         String nomeEmpresa = inReader.readLine();
         System.out.println(":: Digite o site da empresa:");
         String site = inReader.readLine();
         System.out.println(":: Digite o endereço da empresa:");
         String endereco = inReader.readLine();
         System.out.println(":: Digite o telefone da empresa:");
         String telefone = inReader.readLine();
         System.out.println(":: Os dados digitados foram:");
         System.out.println("Nome:"+nomeEmpresa);
         System.out.println("Site:"+site);
         System.out.println("Endereço:"+endereco);
         System.out.println("Telefone:"+telefone);
         String confirma;
         do{ // Pede confirmação do usuário
            System.out.println("Adicionar empresa? [S/n]");
            confirma = inReader.readLine();
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         if (confirma.equals("S") || confirma.equals("s")){
            if (edit) // Edita
               emp.editar(this.idEmpresa, nomeEmpresa, telefone, site, endereco);
            else // Insere
               emp.inserir(nomeEmpresa, telefone, site, endereco);
         }
      }catch (Exception e){
         System.out.println(e);
      }
   }
   
   /**
      Remove uma empresa
   */
   public void removerEmpresa(){
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try{
         System.out.println("Digite o código da empresa");
         String idEmp = inReader.readLine();
         String confirma;
         // Verifica a existência da empresa
         if (!emp.verifica(Integer.parseInt(idEmp))){
            System.out.println("Empresa não encontrada!");
            return;
         }

         // Exibe os dados da empresa e pede confirmação de exclusão
         do{ 
            emp.listar(Integer.parseInt(idEmp));
            System.out.println("Remover empresa? [S/n]");
            confirma = inReader.readLine();
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         // Remoção confirmada
         if (confirma.equals("S") || confirma.equals("s")){ 
            emp.remover(Integer.parseInt(idEmp));
         }
      }catch(Exception e){
         System.out.println(e);
      }
   }
   
   /**
      Insere/edita um logradouro
      @param edit True para edição, False para inserção
   */
   public void inserirLogradouro(boolean edit){
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try {
         String id = "-1";
         // Editar, pede o código do logradouro
         if (edit){
            do{
               System.out.println(":: Digite o código do logradouro:");
               id = inReader.readLine();
            }while (!lg.verifica(Integer.parseInt(id)));
            try{
               lg.listar(Integer.parseInt(id));
            // Logradouro não encontrado
            }catch(Exception e){
               System.out.println("Código inválido! Utilize # listar logradouro para descobrir o código.");
               return;
            }
         }
         // Solicita os dados
         System.out.println(":: Digite o nome do logradouro:");
         String nomeLogradouro = inReader.readLine();
         System.out.println(":: Os dados digitados foram:");
         System.out.println("Nome:"+nomeLogradouro);
         String confirma;
         // Aguarda confirmação para inserir/editar
         do{
            System.out.println("Adicionar logradouro? [S/n]");
            confirma = inReader.readLine();
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         if (confirma.equals("S") || confirma.equals("s")){
            if (edit)
               lg.editar(Integer.parseInt(id), nomeLogradouro);
            else
               lg.inserir(nomeLogradouro);
         }
      }catch (Exception e){
         System.out.println(e);
      }
   }
   
   /**
      Remove um logradouro
   */
   public void removerLogradouro(){
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try{
         // Pede o código do logradouro
         System.out.println("Digite o código do logradouro");
         String idLog = inReader.readLine();
         String confirma;
         // Verifica a existência do logradouro
         if (!lg.verifica(Integer.parseInt(idLog))){
            System.out.println("Logradouro não encontrado!");
            return;
         }
         do{
            // Lista o logradouro
            lg.listar(Integer.parseInt(idLog));
            System.out.println("Remover logradouro? [S/n]");
            confirma = inReader.readLine();
         // Aguarda confirmação para excluir
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         // Exclui
         if (confirma.equals("S") || confirma.equals("s")){
            lg.remover(Integer.parseInt(idLog));
         }
      }catch(Exception e){
         System.out.println(e);
      }
   }   
   
   /**
      Insere ou edita uma linha
   */
   public void inserirLinha(boolean edit){
      // Empresa não selecionada. Exibe mensagem de erro e retorna
      if (!edit && (this.idEmpresa == -1 || !emp.verifica(this.idEmpresa))){
         System.out.println("Selecione uma empresa primeiramente # set empresa codigo");
         return;
      }
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try {
         String id;
         // Editar. Solicita o código da linha
         if (edit){
            do{
               System.out.println(":: Digite o código da linha:");
               id = inReader.readLine();
            }while (!ln.verifica(Integer.parseInt(id)));
            try{
               // Seleciona a linha e a exibe
               this.idLinha = Integer.parseInt(id);
               ln.listar(Integer.parseInt(id));
            // Linha não encontrada
            }catch(Exception e){
               System.out.println("Código inválido! Utilize # listar linhas para descobrir o código.");
               return;
            }
         }
         // Solicita o nome da linha
         System.out.println(":: Digite o nome da linha:");
         String nomeLinha = inReader.readLine();
         if (edit){
            // Mudar uma linha de empresa
            System.out.println(":: Digite o código da empresa:");
            String idEmp = inReader.readLine();
            if (idEmp.contentEquals(""))
               this.idEmpresa = -1;
            else
               this.idEmpresa = Integer.parseInt(idEmp);
         }
         // Exibe os dados digitados
         System.out.println(":: Os dados digitados foram:");
         System.out.println("Código Empresa:"+idEmpresa);
         System.out.println("Nome linha:"+nomeLinha);
         String confirma;
         // Aguarda confirmação para inserção/edição
         do{
            System.out.println("Adicionar linha? [S/n]");
            confirma = inReader.readLine();
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         if (confirma.equals("S") || confirma.equals("s")){
            if (edit)
               ln.editar(this.idLinha, nomeLinha, this.idEmpresa);
            else
               ln.inserir(nomeLinha, this.idEmpresa);
         }
      }catch (Exception e){
         System.out.println(e);
      }
   }
   
   /**
      Remove uma linha
   */
   public void removerLinha(){
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try{
         // Pede o código da linha
         System.out.println("Digite o código da linha");
         String idLin = inReader.readLine();
         String confirma;
         // Verifica se a linha existe
         if (!ln.verifica(Integer.parseInt(idLin))){
            System.out.println("Linha não encontrada!");
            return;
         }
         // Exibe a linha selecionada e aguarda a confirmação para exlusão
         do{
            ln.listar(Integer.parseInt(idLin));
            System.out.println("Remover linha? [S/n]");         
            confirma = inReader.readLine();
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         if (confirma.equals("S") || confirma.equals("s")){
            ln.remover(Integer.parseInt(idLin));
         }
      }catch(Exception e){
         System.out.println(e);
      }
   }
   
   /**
      Insere/edita um horário
   */
   
   public void inserirHorario(boolean edit){
      // Inserção sem linha selecionada não pode ocorrer
      if (this.idLinha == -1 && !edit){
         System.out.println("Selecione uma linha primeiro > set linha codigo");
         return;
      }
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try {
         String id = "-1";
         if (edit){
            do{
            // Solicita o código do horário e tenta listá-lo
               System.out.println(":: Digite o código do horário:");
               id = inReader.readLine();
            }while (!hr.verifica(Integer.parseInt(id)));
            try{
               hr.listar(Integer.parseInt(id));
            // Exibe mensagem de erro se não for possível listar o horário - código digitado não existe
            }catch(Exception e){
               System.out.println("Código inválido! Utilize # listar horarios para descobrir o código.");
               return;
            }
         }
         String hora, min;
         // Solicita o horário
         do{
            System.out.println(":: Digite a hora:");
            hora = inReader.readLine();
         }while(hora.contentEquals(""));
         do{
            System.out.println(":: Digite os minutos:");
            min = inReader.readLine();
         }while(min.contentEquals(""));
         // Exibe os dados digitados
         System.out.println(":: Os dados digitados foram:");
         if (!edit)
            System.out.println("Código linha:"+this.idLinha);
         System.out.println("Horário:"+hora+":"+min);
         String confirma;
         // Pede confirmação para inserção/edição do horário
         do{
            System.out.println("Adicionar horário? [S/n]");
            confirma = inReader.readLine();
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         if (confirma.equals("S") || confirma.equals("s")){
            if(edit)
               hr.editar(Integer.parseInt(id), hora, min);
            else
               hr.inserir(hora, min,idLinha);
         }
      }catch (Exception e){
         System.out.println(e);
      }
   }
   
   
   /**
      Remoção de horário
   */
   public void removerHorario(){
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try{
         // Pede o código do horário
         System.out.println("Digite o código do horário");
         String idHor = inReader.readLine();
         String confirma;
         // Verifica se o horário existe
         if (!hr.verifica(Integer.parseInt(idHor))){
            System.out.println("Horário não encontrado!");
            return;
         }

         // Lista o horário e pede confirmação para exclusão
         do{
            hr.listar(Integer.parseInt(idHor));
            System.out.println("Remover horário? [S/n]");
            confirma = inReader.readLine();
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         if (confirma.equals("S") || confirma.equals("s")){
            hr.remover(Integer.parseInt(idHor));
         }
      }catch(Exception e){
         System.out.println(e);
      }
   }   
   
   /**
      Insere parte de um itinerário em uma linha
   */
   public void inserirItinerario(){
      // Linha não selecionada
      if (this.idLinha == -1){
         System.out.println("Selecione uma linha primeiro # set linha codigo");
         return;
      }
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try {
         System.out.println(":: Digite o código do logradouro:");
         String idLogradouro = inReader.readLine();
         System.out.println(":: Digite a ordem:");
         String ordem = inReader.readLine();
         // Exibe os dados inseridos
         System.out.println(":: Os dados digitados foram:");
         System.out.println("Código linha:"+this.idLinha);
         System.out.println("Código logradouro:"+idLogradouro);
         System.out.println("Ordem:"+ordem);
         String confirma;
         // Aguarda confirmação para inserção
         do{
            System.out.println("Adicionar itinerario? [S/n]");
            confirma = inReader.readLine();
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         if (confirma.equals("S") || confirma.equals("s"))
            it.inserir(this.idLinha, Integer.parseInt(idLogradouro), Integer.parseInt(ordem));
      }catch (Exception e){
         System.out.println(e);
      }
   }
   
   /**
      Remove um itinerário
   */
   public void removerItinerario(){
      BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
      try{
         // Solicita o código da linha
         System.out.println("Digite o código da linha");
         String idLin = inReader.readLine();
         // E a ordem
         System.out.println("Digite a ordem");
         String ordem = inReader.readLine();
         String confirma;
         // Verifica se o itinerário existe
         if (!it.verifica(Integer.parseInt(idLin), Integer.parseInt(ordem))){
            System.out.println("Itinerário não encontrado!");
            return;
         }
         // Lista o itinerário e pede confirmação para remoção
         do{
            it.listar(Integer.parseInt(idLin), Integer.parseInt(ordem));
            System.out.println("Remover itinerário? [S/n]");
            confirma = inReader.readLine();
         }while (!confirma.equals("S") && !confirma.equals("s") && !confirma.equals("n") && !confirma.equals("N"));
         if (confirma.equals("S") || confirma.equals("s")){
            it.remover(Integer.parseInt(idLin), Integer.parseInt(ordem));
         }
      }catch(Exception e){
         System.out.println(e);
      }
   }
   
   public void ajuda(){
      System.out.println("Módulo Administrativo");
      System.out.println("Comandos:");
      System.out.println("* inserir empresa");
      System.out.println("* listar  empresas");
      System.out.println("* remover empresa");
      System.out.println("* editar  empresa");
      System.out.println("* set     empresa : seleciona uma empresa. Necessário para inserção de linhas");
      System.out.println("");
      System.out.println("* inserir linha");
      System.out.println("* listar  linhas");
      System.out.println("* remover linha");
      System.out.println("* editar  linha");
      System.out.println("* set     linha : seleciona uma linha. Necessário para inserção de horários e itinerários");
      System.out.println("");
      System.out.println("* inserir logradouro");
      System.out.println("* listar  logradouros");
      System.out.println("* remover logradouro");
      System.out.println("* editar  logradouro");
      System.out.println("");
      System.out.println("* inserir horário");
      System.out.println("* listar  horário");
      System.out.println("* remover horário");
      System.out.println("* editar  horário");
      System.out.println("");
      System.out.println("* inserir itinerário");
      System.out.println("* listar  itinerário");
      System.out.println("* remover itinerário");      
      System.out.println("");
      System.out.println("* fim : finaliza o programa");      
   }
}

